#!/usr/bin/env python3

import argparse
import sys
import yaml
from pathlib import Path
from tree import SourceNode, SourceIndex
from mparse import create_markdown_parser

# The command line parser
parser = argparse.ArgumentParser(prog='dragonglass')
parser.add_argument('source_dir', help='Source directory (Obsidian vault) for the conversion.')
parser.add_argument('dest_dir', help='Destination directory for the conversion.')
parser.add_argument('-C', '--config', help='Specifies an alternate name for the configuration file.')


class Context:
    def __init__(self):
        self.source_dir = None
        self.config = {}
        self.src_index = None
        self.current_node = None

    def load_config(self, args):
        config_filename = args.config if args.config else ".dragonglass"
        config_path = self.source_dir / config_filename
        if config_path.exists() and config_path.is_file():
            with open(config_path, "r") as f:
                self.config = yaml.full_load(f)

    @property
    def url_prefix(self):
        rc = self.config.get("url_prefix", "/")
        return rc if rc.endswith("/") else rc + '/'


def main():
    args = parser.parse_args()
    context = Context()

    context.source_dir = Path(args.source_dir).resolve()
    if not (context.source_dir.exists() and context.source_dir.is_dir()):
        print(f"{context.source_dir} is not a valid directory")
        return 1

    dest_dir = Path(args.dest_dir).resolve()
    if dest_dir.exists() and not dest_dir.is_dir():
        print(f"{dest_dir} exists but is not a valid directory")
        return 1

    context.load_config(args)

    nodes = SourceNode.generate_list(context.source_dir)
    for node in nodes:
        context.current_node = node
        node.load_metadata(context.source_dir)

    context.src_index = SourceIndex(nodes)

    mdparse = create_markdown_parser(context)
    for node in nodes:
        context.current_node = node
        node.parse_markdown(context.source_dir, mdparse)

    # TEMP
    for node in nodes:
        print(node)
        if node.metadata:
            print(f"Metadata: {node.metadata}")
        if node.text:
            print("----- BEGIN TEXT -----")
            print(node.text)
            print("------ END TEXT ------")

    return 0


if __name__ == '__main__':
    sys.exit(main())
