#!/usr/bin/env python

from urllib.parse import quote as urlquote
import yaml

# The paths that are always to be ignored.
STATIC_IGNORE = [
    '.obsidian',
    '.dragonglass'
]

MARKDOWN_PAT = '*.md'


class SourceNode:
    def __init__(self, root, path, is_dir):
        self._root = root
        self._path = path
        self._is_dir = is_dir
        self._is_md = path.match(MARKDOWN_PAT)
        self.metadata = None
        self.text = None

    def __str__(self):
        return f"SourceNode({self._path}, {self._is_dir}) [is_md={self._is_md}]"

    def link_target(self, prefix="/"):
        xpath = self._path.with_suffix('.html') if self._is_md else self._path
        return urlquote(prefix + xpath.as_posix())

    @classmethod
    def generate_list(cls, source_root):
        nodes = []
        dirs = [source_root]
        while len(dirs) > 0:
            current_dir = dirs.pop(0)
            for child in current_dir.iterdir():
                rchild = child.relative_to(source_root)
                add_me = True
                for pat in STATIC_IGNORE:
                    if rchild.match(pat):
                        add_me = False
                        break
                if add_me:
                    nodes.append(SourceNode(source_root, rchild, child.is_dir()))
                    if child.is_dir():
                        dirs.append(child)
        return nodes

    def load_metadata(self, source_dir):
        if self._is_md and not self._is_dir:
            with open(source_dir / self._path, "r", encoding="utf-8") as f:
                l = f.readline()
                if l == '---\n':
                    metalines = []
                    l = f.readline()
                    while l != '---\n':
                        metalines.append(l)
                        l = f.readline()
                    self.metadata = yaml.full_load(''.join(metalines))

    def parse_markdown(self, source_dir, markdown_parser):
        if self._is_md and not self._is_dir:
            markdown_parser.reset()
            with open(source_dir / self._path, "r", encoding="utf-8") as f:
                self.text = markdown_parser.convert(f.read())


class SourceIndex:
    def __init__(self, nodelist):
        self._byname = {}
        self._byalias = {}
        for node in nodelist:
            if node._is_dir:
                continue
            if node._is_md:
                tmp = node._path.with_suffix('')
                key = tmp.name
                if key not in self._byname:
                    self._byname[key] = node
                self._byname[tmp.as_posix()] = node
                if node.metadata:
                    aliases = node.metadata.get('aliases', [])
                    if isinstance(aliases, list):
                        for alias in aliases:
                            if alias not in self._byalias:
                                self._byalias[alias] = node
            else:
                key = node._path.name
                if key not in self._byname:
                    self._byname[key] = node
                self._byname[node._path.as_posix()] = node

    def lookup(self, reference):
        if reference in self._byname:
            return self._byname[reference], 'NAME'
        elif reference in self._byalias:
            return self._byalias[reference], 'ALIAS'
        else:
            return None, None
